import {action, computed, makeObservable, observable, reaction, runInAction} from "mobx";
import {createOrderValidationSchema, OrderModel} from "../stores/OrderStore";
import {
    Field,
    Fields,
    FrameCandidate,
    FrameFindFilters,
    FrameFindOption,
    FrameOnlyFields,
    FrameTrace,
    IncludedFrameKeys,
    IncludedTraceKeys,
    JOB_TYPES,
    LensUseIndicator,
    prismOptions,
    ProcessedOption,
    RawFrameFindOption,
    TraceKeyMap,
    UnprocessedOption
} from "../stores/_FieldConstants";
import {mixed, number, object, Schema, string} from "yup";
import {t} from "@lingui/macro";
import {errToString, FetchError, schemas, Transport} from "@rx-suite/data";
import {moduleIsInstalled} from "@rx-suite/stores";
import {computedFn, createViewModel, IViewModel} from "mobx-utils";
import {CurrentUserConfig} from "../stores/ConfigStore";
import {customerSelectionStore} from "./useCustomerSelectionStore";

// https://stackoverflow.com/questions/3966484/why-does-modulus-operator-return-fractional-number-in-javascript/31711034#31711034
function floatSafeRemainder(val: number, step: number) {
    const valDecCount = (val.toString().split('.')[1] || '').length;
    const stepDecCount = (step.toString().split('.')[1] || '').length;
    const decCount = valDecCount > stepDecCount ? valDecCount : stepDecCount;
    const valInt = parseInt(val.toFixed(decCount).replace('.', ''));
    const stepInt = parseInt(step.toFixed(decCount).replace('.', ''));
    return (valInt % stepInt) / Math.pow(10, decCount);
}

const generateSchema = (genName: string, f: Field) => {
    const label = f.desc || genName;
    let numberSchema = number().transform((value, original) => {
        if (original === "") {
            return undefined;
        } else {
            return value;
        }
    }).min(Number(f.minValue), t`${label} must be greater than or equal to ${f.minValue}`).max(Number(f.maxValue), t`${label} must be less than or equal to ${f.maxValue}`).test({
        name: "step",
        test: n => n === undefined ? false : floatSafeRemainder(n, Number(f.increment)) === 0,
        message: t`${label} must be increments of ${f.increment}`,
        skipAbsent: true
    }).typeError(t`${label} must be a number`);

    const options = f.options?.map(o => o.code) || [];
    const codes = [...options];

    if (f.mandatory === 1) {
        numberSchema = numberSchema.required(`${genName} is a required field`); // it's a number and required, we can just mark the schema as required
    } else {
        // @ts-ignore the type doesn't handle null, but it's valid
        codes.push("", null); // options are implicitly required because it _must_ equal one of the options. If it's not required, make an empty string a valid option.
    }
    if (f.name == "PRVA1" || f.name == "PRVA2") {
        codes.push(...prismOptions.filter(o => !codes.includes(o)))
    }
    const optionsSchema = string().oneOf(codes, t`${genName} must be one of the following values: ${codes.join(", ")}`).nullable();

    const validateOptions = (n: string, errors: string[]) => {
        try {
            optionsSchema.validateSync(n);
        } catch (e) {
            errors.push(e.message);
        }
    }

    const validateNumber = (n: string, errors: string[]) => {
        try {
            numberSchema.validateSync(n);
        } catch (e) {
            errors.push(e.message);
        }
    }

    return mixed().nullable().test({
        name: '💩',
        test: (n, ctx) => {

            const errors: string[] = [];

            // if the field is a number
            if (f.increment) {
                validateNumber(n, errors);

                if (errors.length) { // if it's not a number
                    if (options.length) { // if it could be an option
                        validateOptions(n, errors);
                        if (errors.length > 1) { // if it's also not an option
                            const errorMessage = errors.join(" or ");

                            return ctx.createError({
                                message: errorMessage
                            })

                        } else { // it's a valid option
                            return true;
                        }
                    }

                    // it's not a number and can't be an option
                    return ctx.createError({
                        message: errors[0]
                    });
                }
            } else if (options.length) { // it must be an option
                validateOptions(n, errors);
                if (errors.length) { // it's not an option
                    return ctx.createError({
                        message: errors[0]
                    })
                }
            }

            // nothing else has errored
            return true
        }
    })
}

export const maybeThrowResponse = async (response: Response) => {
    if (response.ok) {
        return response;
    } else {
        throw new FetchError(await response.json());
    }
}

export const getLeftName = (rightName: string) => {
    if (rightName.startsWith('raw_data.')) {
        return rightName.replace(".rt", ".lt");
    } else if (rightName.startsWith('rt')) {
        return `lt${rightName.slice(2)}`
    } else {
        return `lt${rightName}`;
    }
};

class OrderEntryStore {
    @observable order: OrderModel | null = null;

    @observable fetchingFields = false;
    @observable fetchingFieldsError = '';
    @observable perEye: Field[] = [];
    @observable perJob: Field[] = [];
    @observable leftEyeMode: LensUseIndicator = 'USE';
    @observable rightEyeMode: LensUseIndicator = 'USE';

    @observable showingFrameDialog = false;
    @observable fetchingFrameCandidates = false;
    @observable fetchingFrameCandidatesError = '';
    @observable possibleFrames: FrameCandidate[] = [];

    @observable hasLoadedFrameFindOptions = false;
    @observable fetchingFrameFindOptions = false;
    @observable fetchingFrameFindOptionsError = '';
    @observable frameFindOptions: FrameFindOption[] = [];
    @observable frameFindFilters: FrameFindFilters = {};

    @observable fetchingFrameTrace = false;
    @observable fetchingFrameTraceError = '';
    @observable loadingFrameTraceFile = false;

    @observable upcLookupFieldValue = '';


    viewModel: IViewModel<this> & this;

    constructor() {
        makeObservable(this);
        this.viewModel = createViewModel(this);
    }

    @computed
    get frameTraceFile() {
        return this.order?.viewModel?.trace_file;
    }

    set frameTraceFile(newFile: any) {
        if(this.order) {
            this.order.viewModel.trace_file = newFile;
        }
    }

    @computed
    get frameTrace() {
        return this.order?.viewModel?.frameTrace;
    }

    set frameTrace(newTrace: any) {
        if (this.order) {
            this.order.viewModel.frameTrace = newTrace;
        }
    }

    @action.bound
    clearErrors() {
        this.fetchingFrameTraceError = '';
        this.fetchingFrameCandidatesError = '';
        this.fetchingFieldsError = '';
        this.fetchingFrameFindOptionsError = '';
    }

    @action.bound
    async setTarget(order: OrderModel, readOnly = false) {
        this.order = order;
        this.order.readOnly = readOnly;

        this.clearErrors();

        const currentUserConfig = CurrentUserConfig.get();
         if(order.isNew && !order.isClone) {
            orderEntryStore.clearFrameMeasurements();

             this.order?.nestedAssignment("customer_id", parseInt(customerSelectionStore.currentCustomerId))
            if (currentUserConfig) {
                this.setFrameStatus(currentUserConfig.default_frame_status);
                this.setJobType(currentUserConfig.default_job_type);
            }
        } else if (readOnly){
            this.setJobType(order.job_type);
            if(order.traceID) {
                this.getTrace();
            }
        }

    }

    @action.bound
    async clear() {
        this.order = null;
        await this.clearFields();
    }

    @action.bound
    async clearFields() {
        if (this.order?.readOnly) return;

        this.perEye = [];
        this.perJob = [];
        if(this.order) {
            this.clearFrameMeasurements();
            this.order.raw_data = {};
        }
        await this.resetSchema();
    }

    @action.bound
    async resetSchema() {
        console.log("Resetting schema", this.order);
        schemas["OrderModel"] = createOrderValidationSchema();
        await this.validateOrder();
    }

    @action
    async validateOrder() {
        if(!this.order) return;

        await this.order.validate();
    }

    @computed get hasFields() {
        return !!this.perEye.length || !!this.perJob.length;
    }

    @action.bound
    setRightEyeUsage(val: LensUseIndicator) {
        this.rightEyeMode = val;
    }

    @action.bound
    setLeftEyeUsage(val: LensUseIndicator) {
        this.leftEyeMode = val;
    }

    @computed
    get rightEnabled() {
        return (this.rightEyeMode === 'USE') || (this.rightEyeMode === 'BALANCE');
    }

    @computed
    get rightVisible() {
        return (this.rightEyeMode !== 'HIDE');
    }

    @computed
    get leftEnabled() {
        return (this.leftEyeMode === 'USE') || (this.leftEyeMode === 'BALANCE');
    }

    @computed
    get leftVisible() {
        return (this.leftEyeMode !== 'HIDE');
    }

    @action.bound
    setFrameOnlyFields() {
        this.perJob = FrameOnlyFields;

        if(this.order?.readOnly) return;

        const obj: { [index: string]: Schema } = {};

        FrameOnlyFields.forEach((f: Field) => {
            const genName = f.name;

            if(!this.order?.readOnly) {
                this.order!.raw_data[genName] = f.initialValue;
            }

            obj[genName] = generateSchema(genName, f);
        })

        // @ts-ignore
        schemas["OrderModel"] = createOrderValidationSchema().concat(object().shape({raw_data: object().shape(obj)}));

        this.validateOrder();
    }

    @action
    async getOrderColumns() {
        const response = await Transport.fetch(`/api/orders/order_columns`)
        return await response.json()
    }

    @action
    async getSummaryOrderColumns() {
        const response = await Transport.fetch(`/api/configs/summary_columns`);
        return await response.json();
    }

    @action
    async fetchFields() {
        if(this.fetchingFields) return;

        const order = this.order;
        if(!order) return;

        const styleId = order.viewModel.dig("style_id");
        const frameStatus = order.viewModel.dig("_fstatus");

        try {
            this.fetchingFields = true;
            this.fetchingFieldsError = '';

            const response = await Transport.fetch(`/api/fields?style=${styleId}&frame_status=${frameStatus}`);

            if(order !== this.order) {
                console.warn("Order changed while fetching fields, trying again...");

                runInAction(() => {
                    this.fetchingFields = false;
                    this.tryFetchFields();
                })

                return;
            }

            await maybeThrowResponse(response);
            const json = await response.json() as Fields;

            runInAction(() => {

                const perEye = json.PerEye;
                const perJob = json.PerJob.filter((f: Field) => {
                    if (f.name == "TraceID"){
                        f.name = "traceID";
                    }
                    return f.name !== '_FSTATUS'; // frame status is handled specially
                });

                this.perEye = perEye;
                this.perJob = perJob;

                if(order.readOnly) return;

                const obj: { [index: string]: Schema } = {};

                perEye.forEach((f: Field) => {
                    const eyes = [] as string[];
                    if (this.rightEnabled) {
                        eyes.push('rt');
                    }
                    if (this.leftEnabled) {
                        eyes.push('lt');
                    }
                    eyes.forEach(eye => {
                        const genName = eye + f.name;

                        if (!order.readOnly) {
                            order.raw_data[genName] = f.initialValue;
                        }

                        obj[genName] = generateSchema(genName, f);
                    })
                });

                perJob.forEach((f: Field) => {
                    const genName = f.name;

                    if (!order.readOnly) {
                        order.raw_data[genName] = f.initialValue;
                    }

                    obj[genName] = generateSchema(genName, f);
                })

                console.log("Setting schema")

                if(order.readyOnly) {
                    this.resetSchema();
                } else {
                    // @ts-ignore
                    schemas["OrderModel"] = createOrderValidationSchema().concat(object().shape({raw_data: object().shape(obj)}));
                }

                if (order.isClone){
                    order.clearPersonalizationFields()
                }

                this.validateOrder();
            })
        } catch (e) {
            runInAction(() => this.fetchingFieldsError = errToString(e));
        } finally {
            runInAction(() => this.fetchingFields = false);
        }
    }

    @action
    async tryFetchFields() {
        const styleId = this.order?.viewModel.dig("style_id");

        if(styleId) {
            await this.fetchFields();
        } else if (this.order?.isFrameOnlyJob) {
            this.setFrameOnlyFields();
        } else {
            await this.clearFields();
        }
    }

    maybeGetFrameFinderOptions() {
        if (!this.hasLoadedFrameFindOptions && !this.fetchingFrameFindOptions) {
            this.getFrameFinderOptions();
        }
    }

    @action.bound resetTrace() {
        if(this.order?.readOnly) return;

        IncludedTraceKeys.forEach((value => {
            this.updateRawDataValue(value, undefined);
        }))
        this.fetchingFrameTraceError = '';
        this.frameTrace = null;
    }

    @action.bound clearFrameMeasurements() {
        orderEntryStore.resetTraceFile()
        orderEntryStore.resetTrace();
        orderEntryStore.resetFrameInformation();
    }

    @action.bound
    resetTraceId() {
        this.updateRawDataValue('traceID', null);
    }

    @action.bound resetLensInformation() {
        this.order?.nestedAssignment("quick_code_id",'');
        this.order?.nestedAssignment("style_id",'');
        this.order?.nestedAssignment("style_material_id",'')
        this.order?.nestedAssignment("style_type_id",'');
    }

    @action.bound resetFrameInformation() {
        this.order?.nestedAssignment("fupc",'');
        this.order?.nestedAssignment("fram",'');
        this.order?.nestedAssignment("_fmupc",'');
        this.order?.nestedAssignment("frame_sku",'');

        IncludedFrameKeys.forEach((value => {
            this.updateRawDataValue(value, '');
        }));

    }

    @action.bound
    async getFrameFinderOptions() {
        if (moduleIsInstalled('rx_inventory') && !this.fetchingFrameFindOptions) {
            try {
                runInAction(() => {
                    this.fetchingFrameFindOptions = true;
                    this.fetchingFrameFindOptionsError = '';
                });
                const response = await Transport.fetch('/api/rx_inventory_adapter/frame_search_options');
                await maybeThrowResponse(response);
                const json = await response.json() as RawFrameFindOption[];
                const processed = json.map(o => {
                    return {
                        label: o.label,
                        id: o.id,
                        corresponds_to: o.corresponds_to,
                        options: o.options.map((maybeString: UnprocessedOption) => {
                            if (typeof maybeString === 'string') {
                                return {
                                    label: maybeString,
                                    value: maybeString
                                }
                            } else {
                                return maybeString as ProcessedOption;
                            }

                        })
                    } as FrameFindOption;
                });
                runInAction(() => {
                    this.frameFindOptions = processed;
                    this.hasLoadedFrameFindOptions = true;
                });
            } catch (e) {
                runInAction(() => this.fetchingFrameFindOptionsError = errToString(e));
            } finally {
                runInAction(() => {
                    this.fetchingFrameFindOptions = false;
                });
            }
        }
    }

    frameFindFilterValue = computedFn((id: string) => {
        return this.frameFindFilters[id] || '';
    });

    @action.bound
    handleFrameFindFilterChange(id: string | undefined, value: string) {
        if (id) {
            this.frameFindFilters = {...this.frameFindFilters, [id]: value};
        }
    }

    @action.bound
    handleFrameUPCChange(e: any) {
        this.upcLookupFieldValue = e.target.value
    }

    @action.bound
    showFrameDialog() {
        this.showingFrameDialog = true;
    }

    @action.bound
    hideFrameDialog() {
        this.showingFrameDialog = false;
        this.possibleFrames = [];
    }

    @action.bound
    toggleShowingFrameDialog() {
        this.showingFrameDialog = !this.showingFrameDialog;
    }

    @action.bound
    async maybeCommitUPCLookupField() {
        if (this.upcLookupFieldValue) {
            this.commitUPCLookupField();
        }
    }

    @action.bound
    async commitUPCLookupField() {
        await this.findFramesByUPC(this.upcLookupFieldValue);
        runInAction(() => {
            this.upcLookupFieldValue = '';
        });
    }

    @action.bound
    handleRemoveTraceFile() {
        if (this.order?.viewModel.fupc) {
            this.findFramesByUPC(this.order?.viewModel?.fupc?.toString());
        } else {
            this.resetTrace();
            this.resetTraceFile();
            this.frameTrace = null;
        }
    }

    @action.bound
    handleDrop(files: any) {
        if(!files[0]) {
            return;
        }

        this.loadingFrameTraceFile = true;
        this.resetTraceId();
        this.resetTrace();
        this.resetTraceFile();
        this.setOrderTraceID(null);
        this.frameTraceFile = files[0];

        const reader = new FileReader();
        reader.addEventListener('load', event => {
            if (event.target) {
                this.loadingFrameTraceFile = false;
                if (reader.result){
                    try {
                        this.parseTraceFile(reader.result as string);
                    } catch (err) {
                        this.fetchingFrameTraceError = "Error in uploading trace file"
                    }
                } else {
                    this.fetchingFrameTraceError = "The uploaded trace file is empty"
                }

            }
        });
        reader.readAsText(this.frameTraceFile);

        this.loadingFrameTraceFile = false;
    }

    @action.bound
    async findFramesByUPC(upc: string) {
        if (upc && moduleIsInstalled('rx_inventory') && !this.fetchingFrameCandidates) {
            runInAction(() => {
                this.fetchingFrameCandidates = true;
                this.fetchingFrameCandidatesError = '';
            });
            try {
                const response = await Transport.fetch(`/api/rx_inventory_adapter/frames?upc=${upc}`);
                await maybeThrowResponse(response);
                const json = await response.json() as FrameCandidate[];
                runInAction(() => {
                    this.possibleFrames = json;
                    this.resolveFrameSelection();
                });
            } catch (e) {
                runInAction(() => {
                    this.fetchingFrameCandidatesError = errToString(e);
                });
            } finally {
                runInAction(() => {
                    this.fetchingFrameCandidates = false;
                });
            }
        } else if (!this.fetchingFrameCandidates) {
            runInAction(() => {
                this.possibleFrames = [];
                this.fetchingFrameCandidatesError = '';
            });
        }
    }

    @action.bound
    async findFrames() {
        if (this.hasFrameFilter && moduleIsInstalled('rx_inventory') && !this.fetchingFrameCandidates) {
            runInAction(() => {
                this.fetchingFrameCandidates = true;
                this.fetchingFrameCandidatesError = '';
            });
            try {
                const url = new URL('/api/rx_inventory_adapter/find_frames', window.location.origin);
                Object.entries(this.frameFindFilters).forEach(([id, val]) => {
                    if (val) {
                        url.searchParams.append(id, val);
                    }
                });
                const response = await Transport.fetch(url.toString(), {
                    headers: {
                        Accept: 'application/json'
                    }
                });
                await maybeThrowResponse(response);
                const json = await response.json() as FrameCandidate[];
                runInAction(() => {
                    this.possibleFrames = json;
                });
            } catch (e) {
                runInAction(() => {
                    this.fetchingFrameCandidatesError = errToString(e);
                });
            } finally {
                runInAction(() => {
                    this.fetchingFrameCandidates = false;
                });
            }
        } else if (!this.fetchingFrameCandidates) {
            runInAction(() => {
                this.possibleFrames = [];
                this.fetchingFrameCandidatesError = '';
            });
        }
    }

    @action.bound
    resolveFrameSelection() {
        if (this.possibleFrames.length === 1) {
            this.applyFrame(this.possibleFrames[0]);
        } else if (this.possibleFrames.length > 1) {
            this.showFrameDialog();
        } else {
            console.error('frame error');
            this.fetchingFrameCandidatesError = t`No frames found`;
        }
    }

    @action.bound
    applyFrame(frame: FrameCandidate) {
        if (this.order && frame) {
            this.resetTraceFile();

            const frameFields = {
                _FMUPC: frame.manufacturer_upc || '',
                FRAM: frame.name || '',
                FUPC: frame.upc || '',
                FCOL: frame.color || '',
                _ETYPE: frame.edge_type
            };
            const traceFields = {
                HBOX: frame.a?.toString() || '',
                VBOX: frame.b?.toString() || '',
                FED: frame.ed?.toString() || '',
                DBL: frame.bridge?.toString() || ''
            };

            if (frame.trace) {
                this.order.setFrameData({...frameFields, traceID: frame.trace});
                this.setOrderTraceID(frame.trace);
                this.getTrace();
            } else {
                this.order.setFrameData({
                    ...frameFields,
                    ...traceFields,
                    //circ is not populated if there is no trace
                    CIRC: '',
                    traceID: null
                })
                this.frameTrace = null;
                this.setOrderTraceID(null);
            }

            this.order.frame_sku = frame.id;
            this.order.fram = frame.name || '';
            this.order.fupc = frame.upc || '';
            this.order._fmupc = frame.manufacturer_upc || '';
            this.validateOrder();
        }
        this.hideFrameDialog()
    }

    @action.bound
    async getTrace() {
        if (this.order && !this.fetchingFrameTrace) {
            try {
                this.fetchingFrameTrace = true;
                this.fetchingFrameTraceError = '';
                await this.order.getTrace();
            } catch(e) {
                runInAction(() => {
                    this.fetchingFrameTraceError = errToString(e);
                })
            } finally {
                runInAction(() => {
                    this.fetchingFrameTrace = false;
                    this.validateOrder();
                })
            }
        }
    }

    @action.bound
    setFrameStatus(frameStatus: string) {
        this.order?.nestedAssignment("_fstatus", frameStatus);
    }

    @action.bound
    resetTraceFile() {
        this.frameTraceFile = null;
    }

    @action.bound
    setOrderTraceID(traceID : string | null){
        this.order?.nestedAssignment("trace_id", traceID);
    }

    @action.bound
    setJobType(job_type: string) {
        this.order?.nestedAssignment("job_type", job_type);

        const jobType = JOB_TYPES.find(t => t.value === job_type);

        if (jobType && orderEntryStore.order) {
            if (jobType.value === "A" || jobType.value === "F") {
                this.resetLensInformation();
            } else {
                this.setLeftEyeUsage(jobType.left);
                this.setRightEyeUsage(jobType.right);
            }
        } else {
            console.warn(`Unrecognized job type: ${job_type}`);
        }
    }

    @action.bound
    async parseTraceFile(trace: string | null | undefined) {
        if (trace){
            try {
                const uploadedTrace = {} as FrameTrace;
                const traceArray = (trace.split(/[\s]+/) || []).map(line => {
                    const path = line.split("=") || [];
                    return {key: path[0].toUpperCase(), value: path[1]}
                }).filter( a => IncludedTraceKeys.includes(a.key) || a.key === "TRCFMT");

                const numberOfRadiiPerEye = (traceArray.find( a => a.key == "TRCFMT") || {}).value?.split(";")[1]  || -1
                const allRadiiPoints = traceArray.reduce(
                    (previousValue: number[], currentValue) => {
                        if (currentValue.key == "R") {
                            const points = currentValue.value.split(";") || []
                            const parsedPoints = points.map(a => parseFloat(a)/100);
                            return [...previousValue,...parsedPoints]
                        }
                        return previousValue;
                    },
                    [],
                );

                const singleEyeRadiiPoints = numberOfRadiiPerEye != -1 ? allRadiiPoints.slice(0, parseInt(numberOfRadiiPerEye) - 1 ) : [];

                uploadedTrace["radii"] = singleEyeRadiiPoints;
                traceArray.filter(a => a.key != "R").forEach((traceData) => {
                    const correctedValue = (traceData.value?.split(";") || [])[0];
                    if(correctedValue && this.order){
                        this.updateRawDataValue(traceData.key, correctedValue);
                        uploadedTrace[TraceKeyMap[traceData.key]] = correctedValue
                    }
                });
                runInAction(() => this.frameTrace = uploadedTrace);

            } catch (e) {
                runInAction(() => {
                    this.fetchingFrameTraceError = errToString(e);
                });
            } finally {
                runInAction(() => {
                    if (this.order) {
                        this.validateOrder();
                    }
                });
            }
        }
    }

    @computed
    get hasFrameFilter() {
        return Object.values(this.frameFindFilters).some(f => !!f);
    }

    @computed
    get canPickFrame() {
        return this.order?.viewModel?._fstatus !== "D"
    }

    @computed
    get mustPickFrame() {
        return this.order?.viewModel?._fstatus === "S"
    }

    @computed
    get isUsingFrame() {
        return !!this.order?.viewModel.frame_upc || !!this.order?.viewModel.frame_sku;
    }

    //When order has a frame, we only allow trace upload when the frame not lab-supplied and it has no trace
    @computed
    get shouldLockTraceUploadField() {
        return this.order?.readOnly || this.mustPickFrame ||  (this.isUsingFrame && !!this.order?.traceID) || !!this.order?.traceID;
    }

    @computed
    get shouldLockFrameFields() {
        return this.order?.readOnly || this.mustPickFrame || this.fetchingFrameCandidates || this.showingFrameDialog || this.isUsingFrame;
    }

    @computed
    get shouldLockResizeFields() {
        return this.order?.readOnly || this.isUsingFrame
    }

    @computed
    get canPreviewLens() {
        return !!this.frameTrace
    }
    updateRawDataValue(key : string, value : any){
        this.order?.nestedAssignment(`raw_data.${key}`, value?.toString());
    }
}

const orderEntryStore = new OrderEntryStore();
makeObservable(orderEntryStore);

reaction(() => [orderEntryStore.order?.shipping_method_id, orderEntryStore.order?.readOnly], () => {
    if(orderEntryStore.order?.shipping_method_id == null){
        orderEntryStore.order?.nestedAssignment("shipping_method_id", "");
    }
})

// eslint-disable-next-line @typescript-eslint/ban-ts-comment
// @ts-ignore
if (!window.rxstores) {
    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
    // @ts-ignore
    window.rxstores = {};
}
// eslint-disable-next-line @typescript-eslint/ban-ts-comment
// @ts-ignore
if (!window.rxstores.order_entry_store) {
    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
    // @ts-ignore
    window.rxstores.order_entry_store = orderEntryStore;
}

export const useOrderEntryStore = () => {
    return orderEntryStore;
}
