# == Schema Information
#
# Table name: stock_orders
#
#  id                 :bigint           not null, primary key
#  date_completed     :datetime
#  date_ordered       :datetime
#  order_number       :string(4000)
#  po_number          :string(4000)
#  reference_number   :string(4000)
#  release_number     :string(4000)
#  status             :string(4000)     default("open")
#  created_at         :datetime         not null
#  updated_at         :datetime         not null
#  customer_id        :bigint
#  shipping_method_id :bigint
#  user_id            :bigint
#
# Indexes
#
#  index_stock_orders_on_customer_id         (customer_id)
#  index_stock_orders_on_shipping_method_id  (shipping_method_id)
#
# Foreign Keys
#
#  fk_rails_...  (customer_id => customers.id)
#  fk_rails_...  (shipping_method_id => shipping_methods.id)
#
class StockOrder < ApplicationRecord
  include RxModule::TrimmedNumber

  has_many :stock_order_items, dependent: :destroy
  belongs_to :customer
  belongs_to :shipping_method

  validates_uniqueness_of :reference_number
  validates_presence_of :reference_number
  validate :validate_purchase_order_num, on: :create

  def validate_purchase_order_num
    if StockOrder.where(po_number: self.po_number).exists?
      errors.add(:base, "PO Number is already in use.")
    end
  end

  enum status: {open: "open", closed: "closed"}

  after_initialize do
    self.status ||= "open"
    if po_number.blank? && open?
      RxModule::SerialNumber.for(StockOrder, 'po_number').try_dispense do |num|
        self.po_number = num
      end
    end
  end

  before_destroy do
    if closed?
      errors.add(:base, "Cannot delete an order with a status of closed")
      throw :abort
    end
  end

  def update_items_from_rxuniverse
    offset = 0
    last_response = nil

    loop do
      begin
        response = RxModule::Communicator::Rxuniverse::Orders.line_items(
          id: order_number,
          offset: offset,
          limit: SyncStatus::LIMIT
        )

        break if response[:error].present?

        break if response[:models].empty? || last_response == response

        last_response = response


        response[:models].each do |stock_order_item_json|
          StockOrderItem.load_from_rxuniverse(stock_order_item_json, self)
        end

        offset += SyncStatus::LIMIT

      rescue
        break
      end
    end
  end

  def self.load_from_rxuniverse(data)
    stock_order = StockOrder.find_or_initialize_by(order_number: data[:orderNumber])
    stock_order.assign_attributes(
      date_completed: data[:dateReceived].to_date,
      date_ordered: stock_order.date_ordered.present? ? stock_order.date_ordered : data[:dateReceived],
      po_number: data[:poNumber],
      reference_number: data[:custJobID],
      status: "closed",
      customer: Customer.find_by(number: data[:customerNum]),
      shipping_method: ShippingMethod.find_by(code: data[:shipToMethod]),
    )
    stock_order.save(validate: false)
  end

  def to_json(*args)
    JSON.generate(JSON.parse(super.as_json(*args)).merge(JSON.parse(render_jbuilder_partial)))
  end
end
